<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 20/04/20
 * Time: 10.45
 */

namespace FiloBlu\Rma\Plugin\Model;
use FiloBlu\Rma\Logger\Logger;
use FiloBlu\Rma\Handlers\EasyReturnHandler;
use FiloBlu\Rma\Handlers\RmaItemStatusUpdateHandler;
use FiloBlu\Rma\Handlers\RmaEmailHandler;
use FiloBlu\Rma\Handlers\CancelChangeSizeOrderHandler;
use FiloBlu\Rma\Handlers\CreateChangeSizeOrderHandler;

class RmaModelRmaProcessor
{
    /**
     * @var Logger
     */
    private $logger;
    /**
     * @var EasyReturnHandler
     */
    private $easyReturnHandler;
    /**
     * @var RmaItemStatusUpdateHandler
     */
    private $rmaItemStatusUpdateHandler;
    /**
     * @var RmaEmailHandler
     */
    private $rmaEmailHandler;
    /**
     * @var CancelChangeSizeOrderHandler
     */
    private $cancelChangeSizeOrderHandler;
    /**
     * @var CreateChangeSizeOrderHandler
     */
    private $createChangeSizeOrderHandler;


    /**
     * RmaModelRmaProcessor constructor.
     * @param Logger $logger
     * @param EasyReturnHandler $easyReturnHandler
     * @param RmaItemStatusUpdateHandler $rmaItemStatusUpdateHandler
     * @param RmaEmailHandler $rmaEmailHandler
     * @param CancelChangeSizeOrderHandler $cancelChangeSizeOrderHandler
     * @param CreateChangeSizeOrderHandler $createChangeSizeOrderHandler
     */
    public function __construct(
        Logger $logger,
        EasyReturnHandler $easyReturnHandler,
        RmaItemStatusUpdateHandler $rmaItemStatusUpdateHandler,
        RmaEmailHandler $rmaEmailHandler,
        CancelChangeSizeOrderHandler $cancelChangeSizeOrderHandler,
        CreateChangeSizeOrderHandler $createChangeSizeOrderHandler){
        $this->logger = $logger;
        $this->easyReturnHandler = $easyReturnHandler;
        $this->rmaItemStatusUpdateHandler = $rmaItemStatusUpdateHandler;
        $this->rmaEmailHandler = $rmaEmailHandler;
        $this->cancelChangeSizeOrderHandler = $cancelChangeSizeOrderHandler;
        $this->createChangeSizeOrderHandler = $createChangeSizeOrderHandler;
    }


    /**
     * @param \Magento\Rma\Model\Rma $subject
     * @param $result
     * @return \Magento\Rma\Model\Rma
     */
    public function afterSaveRma(\Magento\Rma\Model\Rma $subject, $result)
    {
        if ($result === false) {
            return $subject;
        }

        try {

            //check if EasyRma is enabled then save EasyRma Fields
            $this->easyReturnHandler->saveEasyReturnFields($subject,$result);

            //check if update status to changeSizeOrder or auto approve rma based on BE configuration
            $this->rmaItemStatusUpdateHandler->shouldUpdateStatus($subject,$result);

            //send email if needed
            $this->rmaEmailHandler->sendRmaEmail($subject);

            //generate changeSizeOrder if needed and send new changeSizeOrder email
            $this->createChangeSizeOrderHandler->checkIfCreateChangeSizeOrder($subject,$result);

            return $result;
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
            return $result;
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
            return $result;
        }
    }


    /**
     * @param \Magento\Rma\Model\Rma $subject
     * @param $result
     * @return mixed
     */
    public function afterClose(\Magento\Rma\Model\Rma $subject, $result){
        try{
            //cancel ChangeSizeOrder on rma cancel action
            $this->cancelChangeSizeOrderHandler->cancelChangeSizeOrder($subject,$result);
            //send email if needed
            $this->rmaEmailHandler->sendRmaEmail($subject);
            return $result;
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
            return $result;
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
            return $result;
        }
    }





}